@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: tor2.c,v 1.17 2004/01/11 17:10:47 markster Exp $
+ * $Id: tor2.c,v 1.19.2.3 2005/01/17 01:58:09 russell Exp $
  */
 
 #include <linux/kernel.h>
@@ -39,6 +39,9 @@
 #else
 #include <linux/zaptel.h>
 #endif
+#ifdef LINUX26
+#include <linux/moduleparam.h>
+#endif
 #define NEED_PCI_IDS
 #include "tor2-hw.h"
 #include "tor2fw.h"
@@ -307,7 +310,7 @@ static void init_spans(struct tor2 *tor)
 			struct zt_chan *mychans = tor->chans[x] + y;
 			sprintf(mychans->name, "Tor2/%d/%d/%d", tor->num, x + 1, y + 1);
 			mychans->sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | ZT_SIG_FXSLS | ZT_SIG_FXSGS | ZT_SIG_FXSKS |
- 									 ZT_SIG_FXOLS | ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS | ZT_SIG_SF;
+ 									 ZT_SIG_FXOLS | ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS | ZT_SIG_SF | ZT_SIG_EM_E1;
 			c = (x * tor->spans[x].channels) + y;
 			mychans->pvt = &tor->tchans[c];
 			mychans->chanpos = y + 1;
@@ -615,7 +618,11 @@ static void __devexit tor2_remove(struct pci_dev *pdev)
 static struct pci_driver tor2_driver = {
 	name: "tormenta2",
 	probe: tor2_probe,
+#ifdef LINUX26
+	remove: __devexit_p(tor2_remove),
+#else
 	remove: tor2_remove,
+#endif
 	id_table: tor2_pci_ids,
 };
 
@@ -1478,10 +1485,18 @@ MODULE_DESCRIPTION("Tormenta 2 PCI Quad T1 or E1 Zaptel Driver");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
+
+#ifdef LINUX26
+module_param(debug, int, 0600);
+module_param(loopback, int, 0600);
+module_param(timingcable, int, 0600);
+module_param(japan, int, 0600);
+#else
 MODULE_PARM(debug, "i");
 MODULE_PARM(loopback, "i");
 MODULE_PARM(timingcable, "i");
 MODULE_PARM(japan, "i");
+#endif
 
 MODULE_DEVICE_TABLE(pci, tor2_pci_ids);
 
