@@ -29,7 +29,6 @@
 #include <linux/interrupt.h>
 #include <linux/ioport.h>
 #include <asm/io.h>
- 
 #ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
 #include "torisa.h"
@@ -37,6 +36,9 @@
 #include <linux/zaptel.h>
 #include <linux/torisa.h>
 #endif
+#ifdef LINUX26
+#include <linux/moduleparam.h>
+#endif
 
 /* Board address offsets (specified in word (not byte) offsets) */
 #define	DDATA 0			/* Data I/O Register */
@@ -1158,11 +1160,20 @@ MODULE_DESCRIPTION("Tormenta ISA Zapata Telephony Driver");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
+
+#ifdef LINUX26
+module_param(prefmaster, int, 0600);
+module_param(base, long, 0600);
+module_param(irq, int, 0600);
+module_param(syncsrc, int, 0600);
+module_param(debug, int, 0600);
+#else
 MODULE_PARM(prefmaster, "i");
 MODULE_PARM(base, "i");
 MODULE_PARM(irq, "i");
 MODULE_PARM(syncsrc, "i");
 MODULE_PARM(debug, "i");
+#endif
 
 module_init(tor_init);
 module_exit(tor_exit);
