@@ -36,6 +36,9 @@
 #else
 #include <linux/zaptel.h>
 #endif
+#ifdef LINUX26
+#include <linux/moduleparam.h>
+#endif
 
 /* Uncomment to enable tasklet handling in the FXO driver.  Not recommended
    in general, but may improve interactive performance */
@@ -972,18 +975,24 @@ static void __devexit wcfxo_remove_one(struct pci_dev *pdev)
 	}
 }
 
-static struct pci_device_id wcfxo_pci_tbl[] __devinitdata = {
+static struct pci_device_id wcfxo_pci_tbl[] = {
 	{ 0xe159, 0x0001, 0x8085, PCI_ANY_ID, 0, 0, (unsigned long) &wcx101p },
 	{ 0xe159, 0x0001, 0x8086, PCI_ANY_ID, 0, 0, (unsigned long) &generic },
+	{ 0xe159, 0x0001, 0x8087, PCI_ANY_ID, 0, 0, (unsigned long) &generic },
 	{ 0x1057, 0x5608, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long) &wcx100p },
+	{ 0 }
 };
 
+MODULE_DEVICE_TABLE (pci, wcfxo_pci_tbl);
+
 static struct pci_driver wcfxo_driver = {
 	name: 	"wcfxo",
 	probe: 	wcfxo_init_one,
+#ifdef LINUX26
+	remove:	__devexit_p(wcfxo_remove_one),
+#else
 	remove:	wcfxo_remove_one,
-	suspend: NULL,
-	resume:	NULL,
+#endif
 	id_table: wcfxo_pci_tbl,
 };
 
@@ -1008,11 +1017,19 @@ static void __exit wcfxo_cleanup(void)
 	pci_unregister_driver(&wcfxo_driver);
 }
 
+#ifdef LINUX26
+module_param(debug, int, 0600);
+module_param(quiet, int, 0600);
+module_param(boost, int, 0600);
+module_param(monitor, int, 0600);
+module_param(opermode, int, 0600);
+#else
 MODULE_PARM(debug, "i");
 MODULE_PARM(quiet, "i");
 MODULE_PARM(boost, "i");
 MODULE_PARM(monitor, "i");
 MODULE_PARM(opermode, "i");
+#endif
 MODULE_DESCRIPTION("Wildcard X100P Zaptel Driver");
 MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
 #ifdef MODULE_LICENSE
