@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wct1xxp.c,v 1.16 2004/01/06 13:26:44 markster Exp $
+ * $Id: wct1xxp.c,v 1.19.2.3 2005/01/17 01:58:09 russell Exp $
  */
 
 #include <linux/kernel.h>
@@ -39,6 +39,9 @@
 #else
 #include <linux/zaptel.h>
 #endif
+#ifdef LINUX26
+#include <linux/moduleparam.h>
+#endif
 
 #define WC_MAX_CARDS	32
 
@@ -786,7 +789,7 @@ static int t1xxp_software_init(struct t1xxp *wc)
 	init_waitqueue_head(&wc->span.maintq);
 	for (x=0;x<wc->span.channels;x++) {
 		sprintf(wc->chans[x].name, "WCT1/%d/%d", wc->num, x + 1);
-		wc->chans[x].sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | 
+		wc->chans[x].sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | ZT_SIG_EM_E1 | 
 				      ZT_SIG_FXSLS | ZT_SIG_FXSGS | 
 				      ZT_SIG_FXSKS | ZT_SIG_FXOLS | 
 				      ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS | ZT_SIG_SF;
@@ -1229,7 +1232,7 @@ static int t1xxp_hardware_init(struct t1xxp *wc)
 	/* Second frame */
 	outl(wc->readdma + ZT_CHUNKSIZE * 32 * 2 - 4, wc->ioaddr + WC_DMARE);	/* End */
 	
-	if (debug) printk("Setting up DMA (write/read = %08x/%08x)\n", wc->writedma, wc->readdma);
+	if (debug) printk("Setting up DMA (write/read = %08lx/%08lx)\n", (long)wc->writedma, (long)wc->readdma);
 
 	/* Check out the controller */
 	if (debug) printk("Controller version: %02x\n", control_get_reg(wc, WC_VERSION));
@@ -1364,14 +1367,21 @@ static void __devexit t1xxp_remove_one(struct pci_dev *pdev)
 	}
 }
 
-static struct pci_device_id t1xxp_pci_tbl[] __devinitdata = {
+static struct pci_device_id t1xxp_pci_tbl[] = {
 	{ 0xe159, 0x0001, 0x6159, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard T100P T1/PRI or E100P E1/PRA Board" },
+	{ 0 }
 };
 
+MODULE_DEVICE_TABLE(pci,t1xxp_pci_tbl);
+
 static struct pci_driver t1xxp_driver = {
 	name: 	"t1xxp",
 	probe: 	t1xxp_init_one,
+#ifdef LINUX26
+	remove:	__devexit_p(t1xxp_remove_one),
+#else
 	remove:	t1xxp_remove_one,
+#endif
 	suspend: NULL,
 	resume:	NULL,
 	id_table: t1xxp_pci_tbl,
@@ -1391,7 +1401,11 @@ static void __exit t1xxp_cleanup(void)
 	pci_unregister_driver(&t1xxp_driver);
 }
 
+#ifdef LINUX26
+module_param(debug, int, 0600);
+#else
 MODULE_PARM(debug, "i");
+#endif
 MODULE_DESCRIPTION("Wildcard T100P/E100P Zaptel Driver");
 MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
 #ifdef MODULE_LICENSE
