@@ -38,6 +38,9 @@
 #else
 #include <linux/zaptel.h>
 #endif
+#ifdef LINUX26
+#include <linux/moduleparam.h>
+#endif
 #include "wct4xxp.h"
 
 /*
@@ -59,6 +62,8 @@ static int timingcable;
 static int highestorder;
 static int t1e1override = -1;
 static int loopback = 0;
+static int alarmdebounce = 0;
+static int noburst = 0;
 
 #ifdef FANCY_ALARM
 static int altab[] = {
@@ -70,6 +75,8 @@ static int altab[] = {
 
 #define FLAG_STARTED (1 << 0)
 #define FLAG_NMF (1 << 1)
+#define FLAG_SENDINGYELLOW (1 << 2)
+
 
 #define	TYPE_T1	1		/* is a T1 card */
 #define	TYPE_E1	2		/* is an E1 card */
@@ -89,7 +96,9 @@ struct t4 {
 	int syncs[4];			/* sync sources */
 	int psyncs[4];			/* span-relative sync sources */
 	int alarmtimer[4];		/* Alarm timer */
+	int redalarms[4];
 	int blinktimer;
+	int alarmcount[4];			/* How much red alarm we've seen */
 #ifdef FANCY_ALARM
 	int alarmpos;
 #endif
@@ -133,8 +142,10 @@ struct t4 {
 	unsigned int passno;	/* number of interrupt passes */
 	char *variety;
 	int last0;		/* for detecting double-missed IRQ */
+	int checktiming;	/* Set >0 to cause the timing source to be checked */
 };
 
+
 static void __set_clear(struct t4 *wc, int span);
 static int t4_startup(struct zt_span *span);
 static int t4_shutdown(struct zt_span *span);
@@ -142,6 +153,7 @@ static int t4_rbsbits(struct zt_chan *chan, int bits);
 static int t4_maint(struct zt_span *span, int cmd);
 static int t4_reset_dma(struct t4 *wc);
 static int t4_ioctl(struct zt_chan *chan, unsigned int cmd, unsigned long data);
+static void __t4_set_timing_source(struct t4 *wc, int unit);
 
 #define WC_RDADDR	0
 #define WC_WRADDR	1
@@ -179,7 +191,7 @@ static inline void __t4_pci_out(struct t4 *wc, const unsigned int addr, const un
 #if 1
 	tmp = wc->membase[addr];
 	if ((value != tmp) && (addr != WC_LEDS) && (addr != WC_LDATA) &&
-		(addr != WC_GPIO))
+		(addr != WC_GPIO) && (addr != WC_INTR))
 		printk("Tried to load %08x into %08x, but got %08x instead\n", value, addr, tmp);
 #endif		
 }
@@ -372,7 +384,7 @@ static int t4_maint(struct zt_span *span, int cmd)
 			t4_framer_out(wc, span->offset, 0x21, 0x40);	/* FMR5: Nothing but RBS mode */
 			break;
 	    default:
-			printk("TE410P: Unknown E1 maint command: %d\n", cmd);
+			printk("TE410P: Unknown T1 maint command: %d\n", cmd);
 			break;
 	   }
     }
@@ -386,8 +398,8 @@ static int t4_rbsbits(struct zt_chan *chan, int bits)
 	struct t4 *wc = chan->pvt;
 	unsigned long flags;
 	
-	spin_lock_irqsave(&wc->reglock, flags);	
 	if(debug) printk("Setting bits to %d on channel %s\n", bits, chan->name);
+	spin_lock_irqsave(&wc->reglock, flags);	
 	k = chan->span->offset;
 	if (wc->spantype[k] == TYPE_E1) { /* do it E1 way */
 		if (chan->chanpos == 16) {
@@ -426,9 +438,9 @@ static int t4_rbsbits(struct zt_chan *chan, int bits)
 		  /* output them to the chip */
 		__t4_framer_out(wc,k,0x70 + b,c); 
 	} 
+	spin_unlock_irqrestore(&wc->reglock, flags);
 	if (debug)
 		printk("Finished setting RBS bits\n");
-	spin_unlock_irqrestore(&wc->reglock, flags);
 	return 0;
 }
 
@@ -462,7 +474,8 @@ static int t4_shutdown(struct zt_span *span)
 		__t4_pci_out(wc, WC_DMACTRL, 0x00000000);
 		/* Acknowledge any pending interrupts */
 		__t4_pci_out(wc, WC_INTR, 0x00000000);
-	}
+		__t4_set_timing_source(wc,4);
+	} else wc->checktiming = 1;
 	spin_unlock_irqrestore(&wc->reglock, flags);
 	if (debug)
 		printk("Span %d (%s) shutdown\n", span->spanno, span->name);
@@ -480,10 +493,13 @@ static int t4_spanconfig(struct zt_span *span, struct zt_lineconfig *lc)
 	span->lineconfig = lc->lineconfig;
 	span->txlevel = lc->lbo;
 	span->rxlevel = 0;
-	span->syncsrc = wc->syncsrc;
+	if (lc->sync < 0)
+		lc->sync = 0;
+	if (lc->sync > 4)
+		lc->sync = 0;
 	
 	/* remove this span number from the current sync sources, if there */
-	for(i = 0; i < 3; i++) {
+	for(i = 0; i < 4; i++) {
 		if (wc->syncs[i] == span->spanno) {
 			wc->syncs[i] = 0;
 			wc->psyncs[i] = 0;
@@ -495,6 +511,7 @@ static int t4_spanconfig(struct zt_span *span, struct zt_lineconfig *lc)
 		wc->syncs[lc->sync - 1] = span->spanno;
 		wc->psyncs[lc->sync - 1] = span->offset + 1;
 	}
+	wc->checktiming = 1;
 	/* If we're already running, then go ahead and apply the changes */
 	if (span->flags & ZT_FLAG_RUNNING)
 		return t4_startup(span);
@@ -572,7 +589,7 @@ static void init_spans(struct t4 *wc)
 			struct zt_chan *mychans = wc->chans[x] + y;
 			sprintf(mychans->name, "TE4/%d/%d/%d", wc->num, x + 1, y + 1);
 			mychans->sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | ZT_SIG_FXSLS | ZT_SIG_FXSGS | ZT_SIG_FXSKS |
-									 ZT_SIG_FXOLS | ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS;
+									 ZT_SIG_FXOLS | ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS | ZT_SIG_EM_E1 | ZT_SIG_DACS_RBS;
 			c = (x * wc->spans[x].channels) + y;
 			mychans->pvt = wc;
 			mychans->chanpos = y + 1;
@@ -607,10 +624,17 @@ static void t4_serial_setup(struct t4 *wc, int unit)
 	t4_framer_out(wc, unit, 0x3f, unit << 1); /* SIC2: No FFS, no center receive eliastic buffer, phase */
 	t4_framer_out(wc, unit, 0x40, 0x04);	/* SIC3: Edges for capture */
 	t4_framer_out(wc, unit, 0x45, 0x00);	/* CMR2: We provide sync and clock for tx and rx. */
-	t4_framer_out(wc, unit, 0x22, 0x00);	/* XC0: Normal operation of Sa-bits */
-	t4_framer_out(wc, unit, 0x23, 0x04);	/* XC1: 0 offset */
-	t4_framer_out(wc, unit, 0x24, 0x04);	/* RC0: Just shy of 1023 */
-	t4_framer_out(wc, unit, 0x25, 0x04);	/* RC1: The rest of RC0 */
+	if (!wc->t1e1) {
+		t4_framer_out(wc, unit, 0x22, 0x03);	/* XC0: Normal operation of Sa-bits */
+		t4_framer_out(wc, unit, 0x23, 0x84);	/* XC1: 0 offset */
+		t4_framer_out(wc, unit, 0x24, 0x03);	/* RC0: Just shy of 1023 */
+		t4_framer_out(wc, unit, 0x25, 0x84);	/* RC1: The rest of RC0 */
+	} else {
+		t4_framer_out(wc, unit, 0x22, 0x00);	/* XC0: Normal operation of Sa-bits */
+		t4_framer_out(wc, unit, 0x23, 0x04);	/* XC1: 0 offset */
+		t4_framer_out(wc, unit, 0x24, 0x04);	/* RC0: Just shy of 1023 */
+		t4_framer_out(wc, unit, 0x25, 0x04);	/* RC1: The rest of RC0 */
+	}
 	
 	/* Configure ports */
 	t4_framer_out(wc, unit, 0x80, 0x00);	/* PC1: SPYR/SPYX input on RPA/XPA */
@@ -622,23 +646,53 @@ static void t4_serial_setup(struct t4 *wc, int unit)
 		printk("Successfully initialized serial bus for unit %d\n", unit);
 }
 
-static void t4_set_timing_source(struct t4 *wc, int unit)
+static void __t4_set_timing_source(struct t4 *wc, int unit)
 {
 	unsigned int timing;
 	int x;
-	timing = 0x34;		/* CMR1: RCLK unit, 8.192 Mhz TCLK, RCLK is 8.192 Mhz */
-	if ((unit > -1) && (unit < 4)) {
-		timing |= (unit << 6);
+	if (unit != wc->syncsrc) {
+		timing = 0x34;		/* CMR1: RCLK unit, 8.192 Mhz TCLK, RCLK is 8.192 Mhz */
+		if ((unit > -1) && (unit < 4)) {
+			timing |= (unit << 6);
+			for (x=0;x<4;x++)  /* set all 4 receive reference clocks to unit */
+				__t4_framer_out(wc, x, 0x44, timing);
+			__t4_pci_out(wc, WC_DMACTRL, wc->dmactrl | (1 << 29));
+		} else {
+			for (x=0;x<4;x++) /* set each receive reference clock to itself */
+				__t4_framer_out(wc, x, 0x44, timing | (x << 6));
+			__t4_pci_out(wc, WC_DMACTRL, wc->dmactrl);
+		}
+		wc->syncsrc = unit;
+		if ((unit < 0) || (unit > 3))
+			unit = 0;
+		else
+			unit++;
+		for (x=0;x<4;x++)
+			wc->spans[x].syncsrc = unit;
 	} else {
-		timing |= 0x1;	/* Use TCLK timing */
+		if (debug)
+			printk("TE410P: Timing source already set to %d\n", unit);
 	}
-	for (x=0;x<4;x++)
-		t4_framer_out(wc, x, 0x44, timing);
-	if ((unit > -1) && (unit < 4))
-		t4_pci_out(wc, WC_DMACTRL, wc->dmactrl | (1 << 29));
-	else
-		t4_pci_out(wc, WC_DMACTRL, wc->dmactrl);
-	printk("TE410P: Timing from source %d\n", unit);
+#if	0
+	printk("wct4xxp: Timing source set to %d\n",unit);
+#endif
+}
+
+static void __t4_set_timing_source_auto(struct t4 *wc)
+{
+	int x;
+	wc->checktiming = 0;
+	for (x=0;x<4;x++) {
+		if (wc->syncs[x]) {
+			if ((wc->spans[wc->syncs[x] - 1].flags & ZT_FLAG_RUNNING) && 
+				!(wc->spans[wc->syncs[x] - 1].alarms & (ZT_ALARM_RED | ZT_ALARM_BLUE) )) {
+					/* Valid timing sourc e*/
+					__t4_set_timing_source(wc, wc->syncs[x] - 1);
+					return;
+			}
+		}
+	}
+	__t4_set_timing_source(wc, 4);
 }
 
 static void __t4_configure_t1(struct t4 *wc, int unit, int lineconfig, int txlevel)
@@ -828,6 +882,8 @@ static int t4_startup(struct zt_span *span)
 			/* enable interrupts */
 		/* Start DMA, enabling DMA interrupts on read only */
 		wc->dmactrl = 0xc0000003 | (1 << 29);
+		if (noburst)
+			wc->dmactrl |= (1 << 26);
 		__t4_pci_out(wc, WC_DMACTRL, wc->dmactrl);
 	}
 
@@ -839,83 +895,6 @@ static int t4_startup(struct zt_span *span)
 	return 0;
 }
 
-#if 0
-static int syncsrc = 0;
-static int syncnum = 0 /* -1 */;
-static int syncspan = 0;
-static spinlock_t synclock = SPIN_LOCK_UNLOCKED;
-
-static int t4_findsync(struct t4 *wc)
-{
-	int i;
-	int x;
-	unsigned long flags;
-	int p;
-	int nonzero;
-	int newsyncsrc = 0;			/* Zaptel span number */
-	int newsyncnum = 0;			/* t4xxp card number */
-	int newsyncspan = 0;		/* span on given t4xxp card */
-	spin_lock_irqsave(&synclock, flags);
-#if 1
-	if (!wc->num) {
-		/* If we're the first card, go through all the motions, up to 8 levels
-		   of sync source */
-		p = 1;
-		while(p < 8) {
-			nonzero = 0;
-			for (x=0;cards[x];x++) {
-				for (i=0;i<4;i++) {
-					if (cards[x]->syncpos[i]) {
-						nonzero = 1;
-						if ((cards[x]->syncpos[i] == p) &&
-						    !(cards[x]->spans[i].alarms & (ZT_ALARM_RED | ZT_ALARM_BLUE | ZT_ALARM_LOOPBACK)) &&
-							(cards[x]->spans[i].flags & ZT_FLAG_RUNNING)) {
-								/* This makes a good sync source */
-								newsyncsrc = cards[x]->spans[i].spanno;
-								newsyncnum = x;
-								newsyncspan = i + 1;
-								/* Jump out */
-								goto found;
-						}
-					}
-				}		
-			}
-			if (nonzero)
-				p++;
-			else 
-				break;
-		}
-found:		
-		if ((syncnum != newsyncnum) || (syncsrc != newsyncsrc) || (newsyncspan != syncspan)) {
-			syncnum = newsyncnum;
-			syncsrc = newsyncsrc;
-			syncspan = newsyncspan;
-			if (debug) printk("New syncnum: %d, syncsrc: %d, syncspan: %d\n", syncnum, syncsrc, syncspan);
-		}
-	}
-#endif	
-	/* update sync src info */
-	if (wc->syncsrc != syncsrc) {
-		wc->syncsrc = syncsrc;
-		/* Update sync sources */
-		for(i = 0; i < 4; i++) {
-			wc->spans[i].syncsrc = wc->syncsrc;
-		}
-		if (syncnum == wc->num) {
-			printk("XXX Set sync span to %d XXX\n", syncspan);
-			if (debug) printk("Card %d, using sync span %d, master\n", wc->num, syncspan);
-			wc->master = 1;	
-		} else {
-			printk("XXX Set timing from external cable XXX\n");
-			wc->master = 0;
-			if (debug) printk("Card %d, using Timing Bus, NOT master\n", wc->num);	
-		}
-	}
-	spin_unlock_irqrestore(&synclock, flags);
-	return 0;
-}
-#endif
-
 static inline void e1_check(struct t4 *wc, int span, int val)
 {
 	if ((wc->spans[span].channels > 24) &&
@@ -943,6 +922,9 @@ static void t4_receiveprep(struct t4 *wc, int irq)
 	int dbl = 0;
 	int x,y,z;
 	unsigned int tmp;
+	int offset=0;
+	if (!wc->t1e1)
+		offset = 4;
 	if (irq & 1) {
 		/* First part */
 		readchunk = wc->readchunk;
@@ -964,7 +946,7 @@ static void t4_receiveprep(struct t4 *wc, int irq)
 	for (x=0;x<ZT_CHUNKSIZE;x++) {
 		for (z=0;z<24;z++) {
 			/* All T1/E1 channels */
-			tmp = readchunk[z+1];
+			tmp = readchunk[z+1+offset];
 			wc->spans[3].chans[z].readchunk[x] = tmp & 0xff;
 			wc->spans[2].chans[z].readchunk[x] = (tmp & 0xff00) >> 8;
 			wc->spans[1].chans[z].readchunk[x] = (tmp & 0xff0000) >> 16;
@@ -1017,6 +999,9 @@ static void t4_transmitprep(struct t4 *wc, int irq)
 	volatile unsigned int *writechunk;
 	int x,y,z;
 	unsigned int tmp;
+	int offset=0;
+	if (!wc->t1e1)
+		offset = 4;
 	if (irq & 1) {
 		/* First part */
 		writechunk = wc->writechunk + 1;
@@ -1036,7 +1021,7 @@ static void t4_transmitprep(struct t4 *wc, int irq)
 				  (wc->spans[2].chans[z].writechunk[x] << 8) |
 				  (wc->spans[1].chans[z].writechunk[x] << 16) |
 				  (wc->spans[0].chans[z].writechunk[x] << 24);
-			writechunk[z] = tmp;
+			writechunk[z+offset] = tmp;
 		}
 		if (wc->t1e1) {
 			for (z=24;z<31;z++) {
@@ -1205,55 +1190,54 @@ static void __t4_check_alarms(struct t4 *wc, int span)
 			alarms |= ZT_ALARM_NOTOPEN;
 	}
 
-	if (c & 0xa0) 
-		alarms |= ZT_ALARM_RED;
+	if (c & 0xa0) {
+		if (wc->alarmcount[span] >= alarmdebounce) 
+			alarms |= ZT_ALARM_RED;
+		else
+			wc->alarmcount[span]++;
+	} else
+		wc->alarmcount[span] = 0;
 	if (c & 0x4)
 		alarms |= ZT_ALARM_BLUE;
+
+	if (((!wc->spans[span].alarms) && alarms) || 
+	    (wc->spans[span].alarms && (!alarms))) 
+		wc->checktiming = 1;
+
 	/* Keep track of recovering */
-	if ((!alarms) && wc->spans[span].alarms)
+	if ((!alarms) && wc->spans[span].alarms) 
 		wc->alarmtimer[span] = ZT_ALARMSETTLE_TIME;
-
-	/* If receiving alarms, go into Yellow alarm state */
-	if (alarms && (!wc->spans[span].alarms)) {
-#if 0
-		printk("Going into yellow alarm\n");
-#endif
-#if 0
-		/* The quadfalc automatically generates yellow alarm */
-		if (wc->spantype[span] == TYPE_E1)
-			__t1_set_reg(wc, 0x21, 0x7f); 
-		else
-			__t1_set_reg(wc, 0x35, 0x11); 
-#endif			
-	}
-
-#if 0
-	if (wc->spans[span].alarms != alarms) {
-		d = __control_get_reg(wc, WC_CLOCK); 
-		start_alarm(wc);
-		if (!(alarms & (ZT_ALARM_RED | ZT_ALARM_BLUE | ZT_ALARM_LOOPBACK)) &&
-		    wc->sync) {
-			/* Use the recieve signalling */
-			wc->spans[span].syncsrc = wc->spans[span].spanno;
-			d |= 1;
-		} else {
-			wc->spans[span].syncsrc = 0;
-			d &= ~1;
-		}
-		__control_set_reg(wc, WC_CLOCK, d);  
-	}
-#endif
 	if (wc->alarmtimer[span])
 		alarms |= ZT_ALARM_RECOVER;
 
+	/* If receiving alarms, go into Yellow alarm state */
+	if (alarms && !(wc->spanflags[span] & FLAG_SENDINGYELLOW)) {
+		unsigned char fmr4;
+#if 1
+		printk("wct4xxp: Setting yellow alarm on span %d\n", span + 1);
+#endif
+		/* We manually do yellow alarm to handle RECOVER and NOTOPEN, otherwise it's auto anyway */
+		fmr4 = __t4_framer_in(wc, span, 0x20);
+		__t4_framer_out(wc, span, 0x20, fmr4 | 0x20);
+		wc->spanflags[span] |= FLAG_SENDINGYELLOW;
+	} else if ((!alarms) && (wc->spanflags[span] & FLAG_SENDINGYELLOW)) {
+		unsigned char fmr4;
+#if 1
+		printk("wct4xxp: Clearing yellow alarm on span %d\n", span + 1);
+#endif
+		/* We manually do yellow alarm to handle RECOVER  */
+		fmr4 = __t4_framer_in(wc, span, 0x20);
+		__t4_framer_out(wc, span, 0x20, fmr4 & ~0x20);
+		wc->spanflags[span] &= ~FLAG_SENDINGYELLOW;
+	}
+
+	/* Re-check the timing source when we enter/leave alarm, not withstanding
+	   yellow alarm */
 	if (c & 0x10)
 		alarms |= ZT_ALARM_YELLOW;
-
 	if (wc->spans[span].mainttimer || wc->spans[span].maintstat) 
 		alarms |= ZT_ALARM_LOOPBACK;
-
 	wc->spans[span].alarms = alarms;
-
 	zt_alarm_notify(&wc->spans[span]);
 }
 
@@ -1403,6 +1387,8 @@ static void t4_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		break;
 	}
 
+	if (wc->checktiming > 0)
+		__t4_set_timing_source_auto(wc);
 	spin_unlock_irqrestore(&wc->reglock, flags);
 #ifdef LINUX26
 	return IRQ_RETVAL(1);
@@ -1424,6 +1410,8 @@ static int t4_reset_dma(struct t4 *wc)
 	t4_pci_out(wc, WC_COUNT, ((ZT_MAX_CHUNKSIZE * 2 * 32 - 1) << 18) | ((ZT_MAX_CHUNKSIZE * 2 * 32 - 1) << 2));
 	t4_pci_out(wc, WC_INTR, 0);
 	wc->dmactrl = 0xc0000003 | (1 << 29);
+	if (noburst)
+		wc->dmactrl |= (1 << 26);
 	t4_pci_out(wc, WC_DMACTRL, wc->dmactrl);
 	return 0;
 }
@@ -1476,6 +1464,7 @@ static int t4_hardware_init(struct t4 *wc)
 static int __devinit t4_launch(struct t4 *wc)
 {
 	int x;
+	unsigned long flags;
 	if (wc->spans[0].flags & ZT_FLAG_REGISTERED)
 		return 0;
 	printk("TE410P: Launching card: %d\n", wc->order);
@@ -1506,11 +1495,10 @@ static int __devinit t4_launch(struct t4 *wc)
 		zt_unregister(&wc->spans[2]);
 		return -1;
 	}
-	if (debug)
-		printk("Setting timing source\n");
-	t4_set_timing_source(wc, 0);
-	if (debug)
-		printk("Timing source selected\n");
+	wc->checktiming = 1;
+	spin_lock_irqsave(&wc->reglock, flags);
+	__t4_set_timing_source(wc,4);
+	spin_unlock_irqrestore(&wc->reglock, flags);
 #ifdef ENABLE_TASKLETS
 	tasklet_init(&wc->t4_tlet, t4_tasklet, (unsigned long)wc);
 #endif
@@ -1720,7 +1708,11 @@ static struct pci_device_id t4_pci_tbl[] __devinitdata =
 static struct pci_driver t4_driver = {
 	name: 	"t4xxp",
 	probe: 	t4_init_one,
+#ifdef LINUX26
+	remove:	__devexit_p(t4_remove_one),
+#else
 	remove:	t4_remove_one,
+#endif
 	suspend: NULL,
 	resume:	NULL,
 	id_table: t4_pci_tbl,
@@ -1746,10 +1738,21 @@ MODULE_DESCRIPTION("TE410P PCI Driver");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
+#ifdef LINUX26
+module_param(debug, int, 0600);
+module_param(loopback, int, 0600);
+module_param(noburst, int, 0600);
+module_param(timingcable, int, 0600);
+module_param(t1e1override, int, 0600);
+module_param(alarmdebounce, int, 0600);
+#else
 MODULE_PARM(debug, "i");
 MODULE_PARM(loopback, "i");
+MODULE_PARM(noburst, "i");
 MODULE_PARM(timingcable, "i");
 MODULE_PARM(t1e1override, "i");
+MODULE_PARM(alarmdebounce, "i");
+#endif
 
 MODULE_DEVICE_TABLE(pci, t4_pci_tbl);
 
