@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wcte11xp.c,v 1.1.2.4 2005/01/17 01:58:09 russell Exp $
+ * $Id: wcte11xp.c,v 1.1.2.5 2005/03/01 14:35:16 russell Exp $
  */
 
 #include <linux/kernel.h>
@@ -1524,7 +1524,10 @@ static void __devexit t1xxp_remove_one(struct pci_dev *pdev)
 }
 
 static struct pci_device_id t1xxp_pci_tbl[] = {
+	{ 0xe159, 0x0001, 0x71fe, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
 	{ 0xe159, 0x0001, 0x79fe, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
+	{ 0xe159, 0x0001, 0x795e, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
+	{ 0xe159, 0x0001, 0x79de, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
 	{ 0 }
 };
 
