@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: zaptel.h,v 1.25 2004/01/02 23:28:52 markster Exp $
+ * $Id: zaptel.h,v 1.38 2004/09/27 19:50:03 markster Exp $
  */
 
 #ifndef _LINUX_ZAPTEL_H
@@ -44,11 +44,20 @@
 #endif
 #include <linux/fs.h>
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+#define LINUX26
+#endif
+
 #include "ecdis.h"
 #include "fasthdlc.h"
 #endif
 #ifdef CONFIG_DEVFS_FS
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #include <linux/devfs_fs_kernel.h>
+#else
+#undef CONFIG_DEVFS_FS
+//#warning "Zaptel doesn't support DEVFS in post 2.4 kernels.  Disabling DEVFS in zaptel"
+#endif
 #endif /* CONFIG_DEVFS_FS */
 #include <linux/ioctl.h>
 
@@ -88,6 +97,8 @@
 
 /* The following are all variations on clear channel */
 
+#define __ZT_SIG_DACS	 (1 << 16)
+
 #define ZT_SIG_CLEAR	 (1 << 7)					/* Clear channel */
 #define ZT_SIG_HDLCRAW	((1 << 8)  | ZT_SIG_CLEAR)	/* Raw unchecked HDLC */
 #define ZT_SIG_HDLCFCS	((1 << 9)  | ZT_SIG_HDLCRAW)	/* HDLC with FCS calculation */
@@ -95,7 +106,9 @@
 #define ZT_SIG_SLAVE	 (1 << 11) 					/* Slave to another channel */
 #define	ZT_SIG_SF	 (1 << 14)			/* Single Freq. tone only, no sig bits */
 #define ZT_SIG_CAS	 (1 << 15)			/* Just get bits */
-#define ZT_SIG_DACS	((1 << 16) | ZT_SIG_CLEAR)	/* Cross connect */
+#define ZT_SIG_DACS	(__ZT_SIG_DACS | ZT_SIG_CLEAR)	/* Cross connect */
+#define ZT_SIG_EM_E1	 (1 << 17)			/* E1 E&M Variation */
+#define ZT_SIG_DACS_RBS	 ((1 << 18) | __ZT_SIG_DACS)	/* Cross connect w/ RBS */
 
 /* tone flag values */
 #define	ZT_REVERSE_RXTONE 1  /* reverse polarity rx tone logic */
@@ -170,6 +183,9 @@ int	starttime;
 int	rxwinktime;
 int	rxflashtime;
 int	debouncetime;
+int	pulsebreaktime;
+int	pulsemaketime;
+int	pulseaftertime;
 } ZT_PARAMS;
 
 typedef struct zt_spaninfo
@@ -557,6 +573,22 @@ char dialstr[ZT_MAX_DTMF_BUF];
  */
 #define	ZT_ONHOOKTRANSFER		_IOW (ZT_CODE, 51, int)
 
+/*
+ * Queue Ping
+ */
+#define ZT_TIMERPING _IOW (ZT_CODE, 42, int) /* Should be 52, but works */
+
+/*
+ * Acknowledge ping
+ */
+#define ZT_TIMERPONG _IOW (ZT_CODE, 53, int)
+
+/*
+ * Set/get signalling freeze
+ */
+#define ZT_SIGFREEZE _IOW (ZT_CODE, 54, int)
+#define ZT_GETSIGFREEZE _IOR (ZT_CODE, 55, int)
+
 /*
  *  60-80 are reserved for private drivers
  *  80-85 are reserved for dynamic span stuff
@@ -641,7 +673,7 @@ struct zt_tone_def {		/* Structure for zone programming */
 	int fac2;		
 	int init_v2_2;		
 	int init_v3_2;
-
+	int modulate;
 
 };
 
@@ -736,6 +768,15 @@ struct zt_tone_def {		/* Structure for zone programming */
 /* Ret value for the beginning of a pulse coming on its way */
 #define ZT_EVENT_PULSE_START 14
 
+/* Timer event -- timer expired */
+#define ZT_EVENT_TIMER_EXPIRED	15
+
+/* Timer event -- ping ready */
+#define ZT_EVENT_TIMER_PING		16
+
+/* Polarity reversal event */
+#define ZT_EVENT_POLARITY  17
+
 #define ZT_EVENT_PULSEDIGIT (1 << 16)	/* This is OR'd with the digit received */
 #define ZT_EVENT_DTMFDIGIT  (1 << 17)	/* Ditto for DTMF */
 
@@ -803,6 +844,9 @@ struct zt_tone_def {		/* Structure for zone programming */
 #define	ZT_DEFAULT_RXWINKTIME 300	/* 300ms longest rx wink (to work with the Atlas) */
 #define	ZT_DEFAULT_RXFLASHTIME 1250	/* 1250ms longest rx flash */
 #define	ZT_DEFAULT_DEBOUNCETIME 600	/* 600ms of FXS GS signalling debounce */
+#define	ZT_DEFAULT_PULSEMAKETIME 50	/* 50 ms of line closed when dial pulsing */
+#define	ZT_DEFAULT_PULSEBREAKTIME 50	/* 50 ms of line open when dial pulsing */
+#define	ZT_DEFAULT_PULSEAFTERTIME 750	/* 750ms between dial pulse digits */
 
 #define	ZT_MINPULSETIME (15 * 8)	/* 15 ms minimum */
 #define	ZT_MAXPULSETIME (150 * 8)	/* 150 ms maximum */
@@ -842,11 +886,16 @@ struct zt_tone_state {
 	int v1_2;
 	int v2_2;
 	int v3_2;
+	int modulate;
 };
 
 #ifdef CONFIG_ZAPATA_NET
 struct zt_hdlc {
+#ifdef LINUX26	
+	struct net_device *netdev;
+#else
 	hdlc_device netdev;
+#endif
 	struct zt_chan *chan;
 };
 #endif
@@ -967,8 +1016,12 @@ struct zt_chan {
 	int		tonep;					/* Current position in tone */
 	struct zt_tone_state ts;		/* Tone state */
 
+	/* Pulse dial stuff */
+	int	pdialcount;			/* pulse dial count */
+
 	/* Ring cadence */
 	int ringcadence[ZT_MAX_CADENCE];
+	int firstcadencepos;				/* Where to restart ring cadence */
 
 	/* Digit string dialing stuff */
 	int		digitmode;			/* What kind of tones are we sending? */
@@ -1027,6 +1080,9 @@ struct zt_chan {
 	int		rxwinktime;  /* rx wink time (ms) */
 	int		rxflashtime; /* rx flash time (ms) */
 	int		debouncetime;  /* FXS GS sig debounce time (ms) */
+	int		pulsebreaktime; /* pulse line open time (ms) */
+	int		pulsemaketime;  /* pulse line closed time (ms) */
+	int		pulseaftertime; /* pulse time between digits (ms) */
 
 	/* RING debounce timer */
 	int	ringdebtimer;
@@ -1111,6 +1167,7 @@ typedef enum {
 #define ZT_FLAG_LINEAR			(1 << 13)	/* Talk to user space in linear */
 #define ZT_FLAG_PPP			(1 << 14)	/* PPP is available */
 #define ZT_FLAG_T1PPP			(1 << 15)
+#define ZT_FLAG_SIGFREEZE		(1 << 16)	/* Freeze signalling */
 
 struct zt_span {
 	spinlock_t lock;
@@ -1322,12 +1379,16 @@ struct zt_tone {
 	int tonesamples;		/* How long to play this tone before 
 					   going to the next (in samples) */
 	struct zt_tone *next;		/* Next tone in this sequence */
+
+	int modulate;
 };
 
 static inline short zt_tone_nextsample(struct zt_tone_state *ts, struct zt_tone *zt)
 {
 	/* follow the curves, return the sum */
 
+	int p;
+
 	ts->v1_1 = ts->v2_1;
 	ts->v2_1 = ts->v3_1;
 	ts->v3_1 = (zt->fac1 * ts->v2_1 >> 15) - ts->v1_1;
@@ -1337,7 +1398,13 @@ static inline short zt_tone_nextsample(struct zt_tone_state *ts, struct zt_tone
 	ts->v3_2 = (zt->fac2 * ts->v2_2 >> 15) - ts->v1_2;
 
 	/* Return top 16 bits */
-	return ts->v3_1 + ts->v3_2;
+	if (!ts->modulate) return ts->v3_1 + ts->v3_2;
+	/* we are modulating */
+	p = ts->v3_2 - 32768;
+	if (p < 0) p = -p;
+	p = ((p * 9) / 10) + 1;
+	return (ts->v3_1 * p) >> 15;
+
 }
 
 static inline short zt_txtone_nextsample(struct zt_chan *ss)
@@ -1372,10 +1439,6 @@ static inline short zt_txtone_nextsample(struct zt_chan *ss)
 
 #endif /* CONFIG_CALC_XLAW */
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-#define LINUX26
-#endif
-
 #endif /* __KERNEL__ */
 
 #endif /* _LINUX_ZAPTEL_H */
