@@ -376,7 +376,7 @@ static int chanconfig(char *keyword, char *args)
 	bzero(chans, sizeof(chans));
 	strtok(args, ":");
 	idle = strtok(NULL, ":");
-	if (!strcasecmp(keyword, "dacs")) {
+	if (!strcasecmp(keyword, "dacs") || !strcasecmp(keyword, "dacsrbs")) {
 		res = parse_channel(idle, &dacschan);
 	}
 	if (!res)
@@ -389,7 +389,7 @@ static int chanconfig(char *keyword, char *args)
 				error("Channel %d already configured as '%s' at line %d\n", x, sig[x], slineno[x]);
 				continue;
 			}
-			if (!strcasecmp(keyword, "dacs") && slineno[dacschan]) {
+			if ((!strcasecmp(keyword, "dacs") || !strcasecmp(keyword, "dacsrbs")) && slineno[dacschan]) {
 				error("DACS Destination channel %d already configured as '%s' at line %d\n", dacschan, sig[dacschan], slineno[dacschan]);
 				continue;
 			} else {
@@ -403,6 +403,9 @@ static int chanconfig(char *keyword, char *args)
 			if (!strcasecmp(keyword, "e&m")) {
 				sig[x] = "E & M";
 				cc[x].sigtype = ZT_SIG_EM;
+			} else if (!strcasecmp(keyword, "e&me1")) {
+				sig[x] = "E & M E1";
+				cc[x].sigtype = ZT_SIG_EM_E1;
 			} else if (!strcasecmp(keyword, "fxsls")) {
 				sig[x] = "FXS Loopstart";
 				cc[x].sigtype = ZT_SIG_FXSLS;
@@ -436,6 +439,16 @@ static int chanconfig(char *keyword, char *args)
 				cc[dacschan].sigtype = ZT_SIG_DACS;
 				sig[dacschan] = "DACS";
 				dacschan++;
+			} else if (!strcasecmp(keyword, "dacsrbs")) {
+				/* Setup channel for monitor */
+				cc[x].idlebits = dacschan;
+				cc[x].sigtype = ZT_SIG_DACS_RBS;
+				sig[x] = "DACS w/ RBS";
+				/* Setup inverse */
+				cc[dacschan].idlebits = x;
+				cc[dacschan].sigtype = ZT_SIG_DACS_RBS;
+				sig[dacschan] = "DACS w/ RBS";
+				dacschan++;
 			} else if (!strcasecmp(keyword, "unused")) {
 				sig[x] = "Unused";
 				cc[x].sigtype = 0;
@@ -571,7 +584,7 @@ static void printconfig()
 			if ((cc[x].sigtype != ZT_SIG_SLAVE) && (cc[x].sigtype)) {
 				configs++;
 				ps = 0;
-				if (cc[x].sigtype == ZT_SIG_DACS)
+				if ((cc[x].sigtype & __ZT_SIG_DACS) == __ZT_SIG_DACS)
 					printf("Channel %02d %s to %02d", x, sig[x], cc[x].idlebits);
 				else {
 					printf("Channel %02d: %s (%s)", x, sig[x], laws[cc[x].deflaw]);
@@ -600,6 +613,7 @@ static struct handler {
 	{ "loadzone", registerzone },
 	{ "defaultzone", defaultzone },
 	{ "e&m", chanconfig },
+	{ "e&me1", chanconfig },
 	{ "fxsls", chanconfig },
 	{ "fxsgs", chanconfig },
 	{ "fxsks", chanconfig },
@@ -616,6 +630,7 @@ static struct handler {
 	{ "unused", chanconfig },
 	{ "cas", chanconfig },
 	{ "dacs", chanconfig },
+	{ "dacsrbs", chanconfig },
 	{ "user", chanconfig },
 	{ "alaw", setlaw },
 	{ "mulaw", setlaw },
@@ -666,7 +681,7 @@ int main(int argc, char *argv[])
 	char *key, *value;
 	int x,found;
 	int fd;
-	while((c = getopt(argc, argv, "hc:vs")) != -1) {
+	while((c = getopt(argc, argv, "thc:vs")) != -1) {
 		switch(c) {
 		case 'c':
 			filename=optarg;
