@@ -22,6 +22,6 @@
 #ifndef _ZTCFG_H
 #define _ZTCFG_H
 
-#define CONFIG_FILENAME "/etc/zaptel.conf"
+#define CONFIG_FILENAME ZAPTEL_CONFIG
 #define MASTER_DEVICE   "/dev/zap/ctl"
 #endif
